<div class="modal fade" id="{{ id }}" tabindex="-1" aria-hidden="true">
    <div class="modal-dialog">
        <div class="modal-content">
            <div class="modal-header">
                <h5 class="modal-title">{% block title %}{% endblock %}</h5>
                <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>
            </div>
            <div class="modal-body">
                {% block content %}{% endblock %}
            </div>
            <div class="modal-footer">
                {% block footer %}
                    <button type="button" class="btn btn-secondary" data-bs-dismiss="modal">Fermer</button>
                {% endblock %}
            </div>
        </div>
    </div>
</div>